@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2015 Derick Rethans                               |
+   | Copyright (c) 2002-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -27,9 +27,6 @@
 
 extern ZEND_DECLARE_MODULE_GLOBALS(xdebug);
 
-static void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa TSRMLS_DC);
-static char* xdebug_func_format(xdebug_func *func TSRMLS_DC);
-
 void xdebug_coverage_line_dtor(void *data)
 {
 	xdebug_coverage_line *line = (xdebug_coverage_line *) data;
@@ -82,13 +79,21 @@ void xdebug_coverage_function_dtor(void *data)
 	xdfree(function);
 }
 
-void xdebug_print_opcode_info(char type, zend_execute_data *execute_data, zend_op *cur_opcode TSRMLS_DC)
+void xdebug_print_opcode_info(char type, zend_execute_data *execute_data, const zend_op *cur_opcode TSRMLS_DC)
 {
+#if PHP_VERSION_ID >= 70000
+	zend_op_array *op_array = &execute_data->func->op_array;
+#else
 	zend_op_array *op_array = execute_data->op_array;
-	char *file = (char *) op_array->filename;
+#endif
+	char *file = (char*) STR_NAME_VAL(op_array->filename);
 	xdebug_func func_info;
 	char *function_name;
+#if PHP_VERSION_ID >= 70000
+	long opnr = execute_data->opline - execute_data->func->op_array.opcodes;
+#else
 	long opnr = execute_data->opline - execute_data->op_array->opcodes;
+#endif
 
 	xdebug_build_fname_from_oparray(&func_info, op_array TSRMLS_CC);
 	function_name = xdebug_func_format(&func_info TSRMLS_CC);
@@ -103,11 +108,15 @@ void xdebug_print_opcode_info(char type, zend_execute_data *execute_data, zend_o
 	xdfree(function_name);
 }
 
-int xdebug_check_branch_entry_handler(ZEND_OPCODE_HANDLER_ARGS)
+int xdebug_check_branch_entry_handler(ZEND_USER_OPCODE_HANDLER_ARGS)
 {
-	if (XG(do_code_coverage)) {
-		zend_op *cur_opcode;
+	if (XG(do_code_coverage) && XG(code_coverage_branch_check)) {
+		const zend_op *cur_opcode;
+#if PHP_VERSION_ID >= 70000
+		cur_opcode = execute_data->opline;
+#else
 		cur_opcode = *EG(opline_ptr);
+#endif
 
 		xdebug_print_opcode_info('G', execute_data, cur_opcode TSRMLS_CC);
 	}
@@ -115,25 +124,29 @@ int xdebug_check_branch_entry_handler(ZEND_OPCODE_HANDLER_ARGS)
 }
 
 #define XDEBUG_OPCODE_OVERRIDE(f) \
-	int xdebug_##f##_handler(ZEND_OPCODE_HANDLER_ARGS) \
+	int xdebug_##f##_handler(ZEND_USER_OPCODE_HANDLER_ARGS) \
 	{ \
-		return xdebug_common_override_handler(ZEND_OPCODE_HANDLER_ARGS_PASSTHRU); \
+		return xdebug_common_override_handler(ZEND_USER_OPCODE_HANDLER_ARGS_PASSTHRU); \
 	}
 
 
-int xdebug_common_override_handler(ZEND_OPCODE_HANDLER_ARGS)
+int xdebug_common_override_handler(ZEND_USER_OPCODE_HANDLER_ARGS)
 {
 	if (XG(do_code_coverage)) {
-		zend_op *cur_opcode;
+		const zend_op *cur_opcode;
 		int      lineno;
 		char    *file;
 
+#if PHP_VERSION_ID >= 70000
+		zend_op_array *op_array = &execute_data->func->op_array;
+		cur_opcode = execute_data->opline;
+		lineno = cur_opcode->lineno;
+#else
 		zend_op_array *op_array = execute_data->op_array;
-
 		cur_opcode = *EG(opline_ptr);
 		lineno = cur_opcode->lineno;
-
-		file = (char *)op_array->filename;
+#endif
+		file = (char*) STR_NAME_VAL(op_array->filename);
 
 		xdebug_print_opcode_info('C', execute_data, cur_opcode TSRMLS_CC);
 		xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
@@ -141,18 +154,70 @@ int xdebug_common_override_handler(ZEND_OPCODE_HANDLER_ARGS)
 	return ZEND_USER_OPCODE_DISPATCH;
 }
 
+#if PHP_VERSION_ID >= 70000
+static int xdebug_is_static_call(const zend_op *cur_opcode, const zend_op *prev_opcode, const zend_op **found_opcode)
+{
+	const zend_op *opcode_ptr;
+
+	opcode_ptr = cur_opcode;
+# if PHP_VERSION_ID >= 70100
+	while (!(opcode_ptr->opcode == ZEND_EXT_STMT) && !((opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_W) || (opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_RW))) {
+		opcode_ptr = opcode_ptr - 1;
+	}
+	if ((opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_W) || (opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_RW)) {
+# else
+	while (!(opcode_ptr->opcode == ZEND_EXT_STMT) && !((opcode_ptr->opcode == ZEND_FETCH_W) || (opcode_ptr->opcode == ZEND_FETCH_RW))) {
+		opcode_ptr = opcode_ptr - 1;
+	}
+	if (((opcode_ptr->opcode == ZEND_FETCH_W) || (opcode_ptr->opcode == ZEND_FETCH_RW)) && opcode_ptr->extended_value == ZEND_FETCH_STATIC_MEMBER) {
+# endif
+		*found_opcode = opcode_ptr;
+		return 1;
+	}
+	return 0;
+}
+#else
+static int xdebug_is_static_call(const zend_op *cur_opcode, const zend_op *prev_opcode, const zend_op **found_opcode)
+{
+	const zend_op *opcode_ptr;
+
+	opcode_ptr = prev_opcode;
+	while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W || opcode_ptr->opcode == ZEND_FETCH_RW) {
+		opcode_ptr = opcode_ptr - 1;
+	}
+	opcode_ptr++;
+
+	if (opcode_ptr->op1_type == IS_CONST && opcode_ptr->extended_value == ZEND_FETCH_STATIC_MEMBER)
+	{
+		*found_opcode = cur_opcode;
+		return 1;
+	}
+	return 0;
+}
+#endif
+
 static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 {
-	zend_op       *cur_opcode, *next_opcode, *prev_opcode = NULL, *opcode_ptr;
+	const zend_op *cur_opcode, *next_opcode, *prev_opcode = NULL, *opcode_ptr;
 	zval          *dimval;
-	int            is_var, cv_len;
+	int            is_var;
+#if PHP_VERSION_ID >= 70000
+	zend_op_array *op_array = &execute_data->func->op_array;
+#else
+	int cv_len;
 	zend_op_array *op_array = execute_data->op_array;
-	xdebug_str     name = {0, 0, NULL};
+#endif
+	xdebug_str     name = XDEBUG_STR_INITIALIZER;
 	int            gohungfound = 0, is_static = 0;
 	char          *zval_value = NULL;
 	xdebug_var_export_options *options;
+	const zend_op *static_opcode_ptr;
 
+#if PHP_VERSION_ID >= 70000
+	cur_opcode = execute_data->opline;
+#else
 	cur_opcode = *EG(opline_ptr);
+#endif
 	next_opcode = cur_opcode + 1;
 	prev_opcode = cur_opcode - 1;
 
@@ -160,18 +225,30 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 			(next_opcode->op1_type == IS_VAR || cur_opcode->op2_type == IS_VAR) &&
 			prev_opcode->opcode == ZEND_FETCH_RW &&
 			prev_opcode->op1_type == IS_CONST &&
+#if PHP_VERSION_ID >= 70000
+			Z_TYPE_P(RT_CONSTANT_EX(op_array->literals, prev_opcode->op1)) == IS_STRING
+#else
 			Z_TYPE_P(prev_opcode->op1.zv) == IS_STRING
+#endif
 	) {
+#if PHP_VERSION_ID >= 70000
+		xdebug_str_add(&name, xdebug_sprintf("$%s", Z_STRVAL_P(RT_CONSTANT_EX(op_array->literals, prev_opcode->op1))), 1);
+#else
 		xdebug_str_add(&name, xdebug_sprintf("$%s", Z_STRVAL_P(prev_opcode->op1.zv)), 1);
+#endif
 	}
 
-	is_static = (prev_opcode->op1_type == IS_CONST && prev_opcode->extended_value == ZEND_FETCH_STATIC_MEMBER);
+	is_static = xdebug_is_static_call(cur_opcode, prev_opcode, &static_opcode_ptr);
 	options = xdebug_var_export_options_from_ini(TSRMLS_C);
 	options->no_decoration = 1;
 
 	if (cur_opcode->op1_type == IS_CV) {
+#if PHP_VERSION_ID >= 70000
+		xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->op1.var)->val), 1);
+#else
 		xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, cur_opcode->op1.var, &cv_len)), 1);
-	} else if (cur_opcode->op1_type == IS_VAR && cur_opcode->opcode == ZEND_ASSIGN && prev_opcode->opcode == ZEND_FETCH_W) {
+#endif
+	} else if (cur_opcode->op1_type == IS_VAR && cur_opcode->opcode == ZEND_ASSIGN && (prev_opcode->opcode == ZEND_FETCH_W || prev_opcode->opcode == ZEND_FETCH_RW)) {
 		if (is_static) {
 			xdebug_str_add(&name, xdebug_sprintf("self::"), 1);
 		} else {
@@ -181,7 +258,7 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	} else if (is_static) { /* todo : see if you can change this and the previous cases around */
 		xdebug_str_add(&name, xdebug_sprintf("self::"), 1 );
 	}
-	if (cur_opcode->opcode >= ZEND_ASSIGN_ADD && cur_opcode->opcode <= ZEND_ASSIGN_BW_XOR
+	if ((cur_opcode->opcode >= ZEND_ASSIGN_ADD && cur_opcode->opcode <= ZEND_ASSIGN_BW_XOR)
 #if PHP_VERSION_ID >= 50600
 		|| cur_opcode->opcode == ZEND_ASSIGN_POW
 #endif
@@ -208,13 +285,23 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	}
 
 	/* Scroll back to start of FETCHES */
+	/* FIXME: See whether we can do this unroll looping only once - in is_static() */
 	gohungfound = 0;
-	opcode_ptr = prev_opcode;
-	while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W || opcode_ptr->opcode == ZEND_FETCH_RW) {
-		opcode_ptr = opcode_ptr - 1;
+#if PHP_VERSION_ID >= 70000
+	if (!is_static) {
+#endif
+		opcode_ptr = prev_opcode;
+		while (opcode_ptr->opcode == ZEND_FETCH_DIM_W || opcode_ptr->opcode == ZEND_FETCH_OBJ_W || opcode_ptr->opcode == ZEND_FETCH_W || opcode_ptr->opcode == ZEND_FETCH_RW) {
+			opcode_ptr = opcode_ptr - 1;
+			gohungfound = 1;
+		}
+		opcode_ptr = opcode_ptr + 1;
+#if PHP_VERSION_ID >= 70000
+	} else { /* if we have a static method, we should already have found the first fetch */
+		opcode_ptr = static_opcode_ptr;
 		gohungfound = 1;
 	}
-	opcode_ptr = opcode_ptr + 1;
+#endif
 
 	if (gohungfound) {
 		do
@@ -223,8 +310,18 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 				xdebug_str_add(&name, "$this", 0);
 			}
 			if (opcode_ptr->op1_type == IS_CV) {
+#if PHP_VERSION_ID >= 70000
+				xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, opcode_ptr->op1.var)->val), 1);
+#else
 				xdebug_str_add(&name, xdebug_sprintf("$%s", zend_get_compiled_variable_name(op_array, opcode_ptr->op1.var, &cv_len)), 1);
+#endif
 			}
+#if PHP_VERSION_ID >= 70100
+			if (opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_W || opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_R || opcode_ptr->opcode == ZEND_FETCH_STATIC_PROP_RW) {
+				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->op1_type, &opcode_ptr->op1, &is_var), 0, options);
+				xdebug_str_add(&name, xdebug_sprintf("%s", zval_value), 1);
+			}
+#endif
 			if (opcode_ptr->opcode == ZEND_FETCH_W) {
 				zval_value = xdebug_get_zval_value(xdebug_get_zval(execute_data, opcode_ptr->op1_type, &opcode_ptr->op1, &is_var), 0, options);
 				xdebug_str_add(&name, xdebug_sprintf("%s", zval_value), 1);
@@ -276,25 +373,33 @@ static char *xdebug_find_var_name(zend_execute_data *execute_data TSRMLS_DC)
 	return name.d;
 }
 
-static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HANDLER_ARGS)
+static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_USER_OPCODE_HANDLER_ARGS)
 {
 	char    *file;
+#if PHP_VERSION_ID >= 70000
+	zend_op_array *op_array = &execute_data->func->op_array;
+#else
 	zend_op_array *op_array = execute_data->op_array;
+#endif
 	int            lineno;
-	zend_op       *cur_opcode, *next_opcode;
+	const zend_op *cur_opcode, *next_opcode;
 	char          *full_varname;
 	zval          *val = NULL;
 	int            is_var;
 	function_stack_entry *fse;
 
+#if PHP_VERSION_ID >= 70000
+	cur_opcode = execute_data->opline;
+#else
 	cur_opcode = *EG(opline_ptr);
+#endif
 	next_opcode = cur_opcode + 1;
-	file = (char *) op_array->filename;
+	file = (char*) STR_NAME_VAL(op_array->filename);
 	lineno = cur_opcode->lineno;
 
 	if (XG(do_code_coverage)) {
 		xdebug_print_opcode_info('=', execute_data, cur_opcode TSRMLS_CC);
-	
+
 		if (do_cc) {
 			xdebug_count_line(file, lineno, 0, 0 TSRMLS_CC);
 		}
@@ -344,9 +449,9 @@ static int xdebug_common_assign_dim_handler(char *op, int do_cc, ZEND_OPCODE_HAN
 }
 
 #define XDEBUG_OPCODE_OVERRIDE_ASSIGN(f,o,cc) \
-	int xdebug_##f##_handler(ZEND_OPCODE_HANDLER_ARGS) \
+	int xdebug_##f##_handler(ZEND_USER_OPCODE_HANDLER_ARGS) \
 	{ \
-		return xdebug_common_assign_dim_handler((o), (cc), ZEND_OPCODE_HANDLER_ARGS_PASSTHRU); \
+		return xdebug_common_assign_dim_handler((o), (cc), ZEND_USER_OPCODE_HANDLER_ARGS_PASSTHRU); \
 	}
 
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign,"=",1)
@@ -375,7 +480,7 @@ XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_bw_xor,"^=",0)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_dim,"=",1)
 XDEBUG_OPCODE_OVERRIDE_ASSIGN(assign_obj,"=",1)
 
-inline void xdebug_count_line(char *filename, int lineno, int executable, int deadcode TSRMLS_DC)
+void xdebug_count_line(char *filename, int lineno, int executable, int deadcode TSRMLS_DC)
 {
 	xdebug_coverage_file *file;
 	xdebug_coverage_line *line;
@@ -426,11 +531,18 @@ static void prefill_from_opcode(char *fn, zend_op opcode, int deadcode TSRMLS_DC
 		&& opcode.opcode != ZEND_OP_DATA
 		&& opcode.opcode != ZEND_ADD_INTERFACE
 		&& opcode.opcode != ZEND_TICKS
+#if PHP_VERSION_ID >= 50500
+		&& opcode.opcode != ZEND_FAST_CALL
+#endif
+#if PHP_VERSION_ID >= 50600
+		&& opcode.opcode != ZEND_RECV_VARIADIC
+#endif
 	) {
 		xdebug_count_line(fn, opcode.lineno, 1, deadcode TSRMLS_CC);
 	}
 }
 
+#if PHP_VERSION_ID < 70000
 static zend_brk_cont_element* xdebug_find_brk_cont(zend_uint nest_levels, int array_offset, zend_op_array *op_array)
 {
 	zend_brk_cont_element *jmp_to;
@@ -445,14 +557,28 @@ static zend_brk_cont_element* xdebug_find_brk_cont(zend_uint nest_levels, int ar
 	} while (--nest_levels > 0);
 	return jmp_to;
 }
+#endif
+
+#define XDEBUG_ZNODE_ELEM(node,var) node.var
+#if ZEND_USE_ABS_JMP_ADDR
+# define XDEBUG_ZNODE_JMP_LINE(node, opline, base)  (int32_t)(((long)((node).jmp_addr) - (long)(base_address)) / sizeof(zend_op))
+#else
+# define XDEBUG_ZNODE_JMP_LINE(node, opline, base)  (int32_t)(((int32_t)((node).jmp_offset) / sizeof(zend_op)) + (opline))
+#endif
 
 static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *jmp2)
 {
+#if PHP_VERSION_ID < 70000 || ZEND_USE_ABS_JMP_ADDR
 	zend_op *base_address = &(opa->opcodes[0]);
+#endif
 
 	zend_op opcode = opa->opcodes[position];
 	if (opcode.opcode == ZEND_JMP) {
-		*jmp1 = ((long) opcode.op1.jmp_addr - (long) base_address) / sizeof(zend_op);
+#if PHP_VERSION_ID >= 70000
+		*jmp1 = XDEBUG_ZNODE_JMP_LINE(opcode.op1, position, base_address);
+#else
+		*jmp1 = ((long) XDEBUG_ZNODE_ELEM(opcode.op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
+#endif
 		return 1;
 	} else if (
 		opcode.opcode == ZEND_JMPZ ||
@@ -461,19 +587,31 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 		opcode.opcode == ZEND_JMPNZ_EX
 	) {
 		*jmp1 = position + 1;
-		*jmp2 = ((long) opcode.op2.jmp_addr - (long) base_address) / sizeof(zend_op);
+#if PHP_VERSION_ID >= 70000
+		*jmp2 = XDEBUG_ZNODE_JMP_LINE(opcode.op2, position, base_address);
+#else
+		*jmp2 = ((long) XDEBUG_ZNODE_ELEM(opcode.op2, jmp_addr) - (long) base_address) / sizeof(zend_op);
+#endif
 		return 1;
 	} else if (opcode.opcode == ZEND_JMPZNZ) {
-		*jmp1 = opcode.op2.opline_num;
+#if PHP_VERSION_ID >= 70000
+		*jmp1 = XDEBUG_ZNODE_JMP_LINE(opcode.op2, position, base_address);
+		*jmp2 = (int32_t) (position + ((int32_t)opcode.extended_value / sizeof(zend_op)));
+#else
+		*jmp1 = XDEBUG_ZNODE_ELEM(opcode.op2, opline_num);
 		*jmp2 = opcode.extended_value;
+#endif
 		return 1;
+#if PHP_VERSION_ID < 70000
 	} else if (opcode.opcode == ZEND_BRK || opcode.opcode == ZEND_CONT) {
 		zend_brk_cont_element *el;
 
 		if (opcode.op2_type == IS_CONST
-		    && opcode.op1.jmp_addr != (zend_op*) 0xFFFFFFFF
+#ifdef ZEND_ENGINE_2
+		    && XDEBUG_ZNODE_ELEM(opcode.op1, jmp_addr) != (zend_op*) 0xFFFFFFFF
+#endif
 		) {
-			el = xdebug_find_brk_cont(Z_LVAL_P(opcode.op2.zv), opcode.op1.opline_num, opa);
+			el = xdebug_find_brk_cont(Z_LVAL_P(opcode.op2.zv), XDEBUG_ZNODE_ELEM(opcode.op1, opline_num), opa);
 			if (el) {
 				*jmp1 = opcode.opcode == ZEND_BRK ? el->brk : el->cont;
 				return 1;
@@ -482,14 +620,31 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 				return 0;
 			}
 		}
-	} else if (opcode.opcode == ZEND_FE_RESET || opcode.opcode == ZEND_FE_FETCH) {
+#endif
+#if PHP_VERSION_ID >= 70000
+	} else if (opcode.opcode == ZEND_FE_FETCH_R || opcode.opcode == ZEND_FE_FETCH_RW) {
 		*jmp1 = position + 1;
-		*jmp2 = opcode.op2.opline_num;
+		*jmp2 = position + (opcode.extended_value / sizeof(zend_op));
+		return 1;
+	} else if (opcode.opcode == ZEND_FE_RESET_R || opcode.opcode == ZEND_FE_RESET_RW) {
+#else
+	} else if (opcode.opcode == ZEND_FE_RESET || opcode.opcode == ZEND_FE_FETCH) {
+#endif
+		*jmp1 = position + 1;
+#if PHP_VERSION_ID >= 70000
+		*jmp2 = XDEBUG_ZNODE_JMP_LINE(opcode.op2, position, base_address);
+#else
+		*jmp2 = XDEBUG_ZNODE_ELEM(opcode.op2, opline_num);
+#endif
 		return 1;
 	} else if (opcode.opcode == ZEND_CATCH) {
 		*jmp1 = position + 1;
 		if (!opcode.result.num) {
+#if PHP_VERSION_ID >= 70100
+			*jmp2 = position + (opcode.extended_value / sizeof(zend_op));
+#else
 			*jmp2 = opcode.extended_value;
+#endif
 			if (*jmp2 == *jmp1) {
 				*jmp2 = XDEBUG_JMP_NOT_SET;
 			}
@@ -498,9 +653,35 @@ static int xdebug_find_jump(zend_op_array *opa, unsigned int position, long *jmp
 		}
 		return 1;
 	} else if (opcode.opcode == ZEND_GOTO) {
-		*jmp1 = ((long) opcode.op1.jmp_addr - (long) base_address) / sizeof(zend_op);
+#if PHP_VERSION_ID >= 70000
+		*jmp1 = XDEBUG_ZNODE_JMP_LINE(opcode.op1, position, base_address);
+#else
+		*jmp1 = ((long) XDEBUG_ZNODE_ELEM(opcode.op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
+#endif
 		return 1;
-	} else if (opcode.opcode == ZEND_EXIT || opcode.opcode == ZEND_THROW || opcode.opcode == ZEND_RETURN) {
+
+#if PHP_VERSION_ID >= 50500
+	} else if (opcode.opcode == ZEND_FAST_CALL) {
+#if PHP_VERSION_ID >= 70000
+		*jmp1 = XDEBUG_ZNODE_JMP_LINE(opcode.op1, position, base_address);
+#else
+		*jmp1 = ((long) XDEBUG_ZNODE_ELEM(opcode.op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
+#endif
+		*jmp2 = position + 1;
+		return 1;
+	} else if (opcode.opcode == ZEND_FAST_RET) {
+		*jmp1 = XDEBUG_JMP_EXIT;
+		return 1;
+#endif
+
+	} else if (
+#if PHP_VERSION_ID >= 50500
+		opcode.opcode == ZEND_GENERATOR_RETURN ||
+#endif
+		opcode.opcode == ZEND_EXIT ||
+		opcode.opcode == ZEND_THROW ||
+		opcode.opcode == ZEND_RETURN
+	) {
 		*jmp1 = XDEBUG_JMP_EXIT;
 		return 1;
 	}
@@ -620,23 +801,23 @@ static void xdebug_analyse_oparray(zend_op_array *opa, xdebug_set *set, xdebug_b
 	}
 }
 
-static void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa TSRMLS_DC)
+void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa TSRMLS_DC)
 {
 	int closure = 0;
 
 	memset(tmp, 0, sizeof(xdebug_func));
 
 	if (opa->function_name) {
-		if (strcmp(opa->function_name, "{closure}") == 0) {
+		if (strcmp(STR_NAME_VAL(opa->function_name), "{closure}") == 0) {
 			tmp->function = xdebug_sprintf(
 				"{closure:%s:%d-%d}",
-				opa->filename,
+				STR_NAME_VAL(opa->filename),
 				opa->line_start,
 				opa->line_end
 			);
 			closure = 1;
 		} else {
-			tmp->function = xdstrdup(opa->function_name);
+			tmp->function = xdstrdup(STR_NAME_VAL(opa->function_name));
 		}
 	} else {
 		tmp->function = xdstrdup("{main}");
@@ -644,13 +825,13 @@ static void xdebug_build_fname_from_oparray(xdebug_func *tmp, zend_op_array *opa
 
 	if (opa->scope && !closure) {
 		tmp->type = XFUNC_MEMBER;
-		tmp->class = xdstrdup(opa->scope->name);
+		tmp->class = xdstrdup(STR_NAME_VAL(opa->scope->name));
 	} else {
 		tmp->type = XFUNC_NORMAL;
 	}
 }
 
-static char* xdebug_func_format(xdebug_func *func TSRMLS_DC)
+char* xdebug_func_format(xdebug_func *func TSRMLS_DC)
 {
 	switch (func->type) {
 		case XFUNC_NORMAL:
@@ -672,9 +853,13 @@ static void prefill_from_oparray(char *filename, zend_op_array *op_array TSRMLS_
 
 	/* Check for abstract methods and simply return from this function in those
 	 * cases. */
+#if PHP_VERSION_ID >= 70000
+	if (op_array->fn_flags & ZEND_ACC_ABSTRACT) {
+#else
 	if (op_array->last >= 3 && op_array->opcodes[op_array->last - 3].opcode == ZEND_RAISE_ABSTRACT_ERROR) {
+#endif
 		return;
-	}	
+	}
 
 	/* Run dead code analysis if requested */
 	if (XG(code_coverage_dead_code_analysis) && (op_array->fn_flags & ZEND_ACC_DONE_PASS_TWO)) {
@@ -717,32 +902,56 @@ static void prefill_from_oparray(char *filename, zend_op_array *op_array TSRMLS_
 	}
 }
 
+#if PHP_VERSION_ID >= 70000
+static int prefill_from_function_table(zend_op_array *opa)
+#else
 static int prefill_from_function_table(zend_op_array *opa TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
+#endif
 {
-	char *new_filename;
-
-	new_filename = va_arg(args, char*);
 	if (opa->type == ZEND_USER_FUNCTION) {
 		if ((long) opa->reserved[XG(dead_code_analysis_tracker_offset)] < XG(dead_code_last_start_id)) {
-			prefill_from_oparray((char *) opa->filename, opa TSRMLS_CC);
+			prefill_from_oparray((char*) STR_NAME_VAL(opa->filename), opa TSRMLS_CC);
 		}
 	}
 
 	return ZEND_HASH_APPLY_KEEP;
 }
 
+#if PHP_VERSION_ID >= 70000
+static int prefill_from_class_table(zend_class_entry *class_entry)
+#else
 static int prefill_from_class_table(zend_class_entry **class_entry TSRMLS_DC, int num_args, va_list args, zend_hash_key *hash_key)
+#endif
 {
+#if PHP_VERSION_ID < 70000
 	char *new_filename;
+#endif
 	zend_class_entry *ce;
 
+#if PHP_VERSION_ID >= 70000
+	ce = class_entry;
+#else
 	ce = *class_entry;
+#endif
 
+#if PHP_VERSION_ID < 70000
 	new_filename = va_arg(args, char*);
+#endif
 	if (ce->type == ZEND_USER_CLASS) {
 		if (!(ce->ce_flags & ZEND_XDEBUG_VISITED)) {
+#if PHP_VERSION_ID >= 70000
+			zend_op_array *val;
+#endif
 			ce->ce_flags |= ZEND_XDEBUG_VISITED;
+#if PHP_VERSION_ID >= 70000
+			ZEND_HASH_INC_APPLY_COUNT(&ce->function_table);
+			ZEND_HASH_FOREACH_PTR(&ce->function_table, val) {
+				prefill_from_function_table(val);
+			} ZEND_HASH_FOREACH_END();
+			ZEND_HASH_DEC_APPLY_COUNT(&ce->function_table);
+#else
 			zend_hash_apply_with_arguments(&ce->function_table TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, new_filename);
+#endif
 		}
 	}
 
@@ -751,36 +960,52 @@ static int prefill_from_class_table(zend_class_entry **class_entry TSRMLS_DC, in
 
 void xdebug_prefill_code_coverage(zend_op_array *op_array TSRMLS_DC)
 {
+#if PHP_VERSION_ID >= 70000
+	zend_op_array    *function_op_array;
+	zend_class_entry *class_entry;
+#endif
+
 	if ((long) op_array->reserved[XG(dead_code_analysis_tracker_offset)] < XG(dead_code_last_start_id)) {
-		prefill_from_oparray((char *) op_array->filename, op_array TSRMLS_CC);
+		prefill_from_oparray((char*) STR_NAME_VAL(op_array->filename), op_array TSRMLS_CC);
 	}
 
-	zend_hash_apply_with_arguments(CG(function_table)  TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, op_array->filename);
-	zend_hash_apply_with_arguments(CG(class_table) TSRMLS_CC, (apply_func_args_t) prefill_from_class_table, 1, op_array->filename);
+#if PHP_VERSION_ID >= 70000
+	ZEND_HASH_INC_APPLY_COUNT(CG(function_table));
+	ZEND_HASH_FOREACH_PTR(CG(function_table), function_op_array) {
+		prefill_from_function_table(function_op_array);
+	} ZEND_HASH_FOREACH_END();
+	ZEND_HASH_DEC_APPLY_COUNT(CG(function_table));
+
+	ZEND_HASH_INC_APPLY_COUNT(CG(class_table));
+	ZEND_HASH_FOREACH_PTR(CG(class_table), class_entry) {
+		prefill_from_class_table(class_entry);
+	} ZEND_HASH_FOREACH_END();
+	ZEND_HASH_DEC_APPLY_COUNT(CG(class_table));
+#else
+	zend_hash_apply_with_arguments(CG(function_table)  TSRMLS_CC, (apply_func_args_t) prefill_from_function_table, 1, STR_NAME_VAL(op_array->filename));
+	zend_hash_apply_with_arguments(CG(class_table) TSRMLS_CC, (apply_func_args_t) prefill_from_class_table, 1, STR_NAME_VAL(op_array->filename));
+#endif
 }
 
-void xdebug_code_coverage_start_of_function(zend_op_array *op_array TSRMLS_DC)
+void xdebug_code_coverage_start_of_function(zend_op_array *op_array, char *function_name TSRMLS_DC)
 {
 	xdebug_path *path = xdebug_path_new(NULL);
 
 	xdebug_prefill_code_coverage(op_array TSRMLS_CC);
 	xdebug_path_info_add_path_for_level(XG(paths_stack), path, XG(level) TSRMLS_CC);
 
-	if (XG(branches).size == 0 || XG(level) > XG(branches).size) {
-		XG(branches).size += 32;
+	if (XG(branches).size == 0 || XG(level) >= XG(branches).size) {
+		XG(branches).size = XG(level) + 32;
 		XG(branches).last_branch_nr = realloc(XG(branches).last_branch_nr, sizeof(int) * XG(branches.size));
 	}
 
 	XG(branches).last_branch_nr[XG(level)] = -1;
 }
 
-void xdebug_code_coverage_end_of_function(zend_op_array *op_array TSRMLS_DC)
+void xdebug_code_coverage_end_of_function(zend_op_array *op_array, char *file_name, char *function_name TSRMLS_DC)
 {
-	xdebug_str str = { 0, 0, NULL };
+	xdebug_str str = XDEBUG_STR_INITIALIZER;
 	xdebug_path *path = xdebug_path_info_get_path_for_level(XG(paths_stack), XG(level) TSRMLS_CC);
-	char *file = (char *) op_array->filename;
-	xdebug_func func_info;
-	char *function_name;
 
 	if (!path) {
 		return;
@@ -788,19 +1013,8 @@ void xdebug_code_coverage_end_of_function(zend_op_array *op_array TSRMLS_DC)
 
 	xdebug_create_key_for_path(path, &str);
 
-	xdebug_build_fname_from_oparray(&func_info, op_array TSRMLS_CC);
-	function_name = xdebug_func_format(&func_info TSRMLS_CC);
+	xdebug_branch_info_mark_end_of_function_reached(file_name, function_name, str.d, str.l TSRMLS_CC);
 
-	if (func_info.class) {
-		xdfree(func_info.class);
-	}
-	if (func_info.function) {
-		xdfree(func_info.function);
-	}
-
-	xdebug_branch_info_mark_end_of_function_reached(file, function_name, str.d, str.l TSRMLS_CC);
-
-	xdfree(function_name);
 	xdfree(str.d);
 
 	if (path) {
@@ -810,7 +1024,7 @@ void xdebug_code_coverage_end_of_function(zend_op_array *op_array TSRMLS_DC)
 
 PHP_FUNCTION(xdebug_start_code_coverage)
 {
-	long options = 0;
+	zppLONG options = 0;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &options) == FAILURE) {
 		return;
@@ -833,7 +1047,7 @@ PHP_FUNCTION(xdebug_start_code_coverage)
 
 PHP_FUNCTION(xdebug_stop_code_coverage)
 {
-	long cleanup = 1;
+	zppLONG cleanup = 1;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &cleanup) == FAILURE) {
 		return;
@@ -859,8 +1073,13 @@ PHP_FUNCTION(xdebug_stop_code_coverage)
 
 static int xdebug_lineno_cmp(const void *a, const void *b TSRMLS_DC)
 {
+#if PHP_VERSION_ID >= 70000
+	Bucket *f = (Bucket *) a;
+	Bucket *s = (Bucket *) b;
+#else
 	Bucket *f = *((Bucket **) a);
 	Bucket *s = *((Bucket **) b);
+#endif
 
 	if (f->h < s->h) {
 		return -1;
@@ -889,12 +1108,12 @@ static void add_branches(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC
 	zval *branches, *branch, *out, *out_hit;
 	unsigned int i;
 
-	MAKE_STD_ZVAL(branches);
+	XDEBUG_MAKE_STD_ZVAL(branches);
 	array_init(branches);
 
 	for (i = 0; i < branch_info->starts->size; i++) {
 		if (xdebug_set_in(branch_info->starts, i)) {
-			MAKE_STD_ZVAL(branch);
+			XDEBUG_MAKE_STD_ZVAL(branch);
 			array_init(branch);
 			add_assoc_long(branch, "op_start", i);
 			add_assoc_long(branch, "op_end", branch_info->branches[i].end_op);
@@ -903,7 +1122,7 @@ static void add_branches(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC
 
 			add_assoc_long(branch, "hit", branch_info->branches[i].hit);
 
-			MAKE_STD_ZVAL(out);
+			XDEBUG_MAKE_STD_ZVAL(out);
 			array_init(out);
 			if (branch_info->branches[i].out[0]) {
 				add_index_long(out, 0, branch_info->branches[i].out[0]);
@@ -913,7 +1132,7 @@ static void add_branches(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC
 			}
 			add_assoc_zval(branch, "out", out);
 
-			MAKE_STD_ZVAL(out_hit);
+			XDEBUG_MAKE_STD_ZVAL(out_hit);
 			array_init(out_hit);
 			if (branch_info->branches[i].out[0]) {
 				add_index_long(out_hit, 0, branch_info->branches[i].out_hit[0]);
@@ -924,10 +1143,19 @@ static void add_branches(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC
 			add_assoc_zval(branch, "out_hit", out_hit);
 
 			add_index_zval(branches, i, branch);
+#if PHP_VERSION_ID >= 70000
+			efree(out_hit);
+			efree(out);
+			efree(branch);
+#endif
 		}
 	}
 
-	add_assoc_zval_ex(retval, "branches", 9, branches);
+	add_assoc_zval_ex(retval, "branches", HASH_KEY_SIZEOF("branches"), branches);
+
+#if PHP_VERSION_ID >= 70000
+	efree(branches);
+#endif
 }
 
 static void add_paths(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC)
@@ -935,14 +1163,14 @@ static void add_paths(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC)
 	zval *paths, *path, *path_container;
 	unsigned int i, j;
 
-	MAKE_STD_ZVAL(paths);
+	XDEBUG_MAKE_STD_ZVAL(paths);
 	array_init(paths);
 
 	for (i = 0; i < branch_info->path_info.paths_count; i++) {
-		MAKE_STD_ZVAL(path);
+		XDEBUG_MAKE_STD_ZVAL(path);
 		array_init(path);
 
-		MAKE_STD_ZVAL(path_container);
+		XDEBUG_MAKE_STD_ZVAL(path_container);
 		array_init(path_container);
 
 		for (j = 0; j < branch_info->path_info.paths[i]->elements_count; j++) {
@@ -953,9 +1181,18 @@ static void add_paths(zval *retval, xdebug_branch_info *branch_info TSRMLS_DC)
 		add_assoc_long(path_container, "hit", branch_info->path_info.paths[i]->hit);
 
 		add_next_index_zval(paths, path_container);
+
+#if PHP_VERSION_ID >= 70000
+		efree(path_container);
+		efree(path);
+#endif
 	}
 
-	add_assoc_zval_ex(retval, "paths", 6, paths);
+	add_assoc_zval_ex(retval, "paths", HASH_KEY_SIZEOF("paths"), paths);
+
+#if PHP_VERSION_ID >= 70000
+	efree(paths);
+#endif
 }
 
 static void add_cc_function(void *ret, xdebug_hash_element *e)
@@ -965,7 +1202,7 @@ static void add_cc_function(void *ret, xdebug_hash_element *e)
 	zval                     *function_info;
 	TSRMLS_FETCH();
 
-	MAKE_STD_ZVAL(function_info);
+	XDEBUG_MAKE_STD_ZVAL(function_info);
 	array_init(function_info);
 
 	if (function->branch_info) {
@@ -973,7 +1210,11 @@ static void add_cc_function(void *ret, xdebug_hash_element *e)
 		add_paths(function_info, function->branch_info TSRMLS_CC);
 	}
 
-	add_assoc_zval_ex(retval, function->name, strlen(function->name) + 1, function_info);
+	add_assoc_zval_ex(retval, function->name, HASH_KEY_STRLEN(function->name), function_info);
+
+#if PHP_VERSION_ID >= 70000
+	efree(function_info);
+#endif
 }
 
 static void add_file(void *ret, xdebug_hash_element *e)
@@ -985,32 +1226,44 @@ static void add_file(void *ret, xdebug_hash_element *e)
 	TSRMLS_FETCH();
 
 	/* Add all the lines */
-	MAKE_STD_ZVAL(lines);
+	XDEBUG_MAKE_STD_ZVAL(lines);
 	array_init(lines);
 
 	xdebug_hash_apply(file->lines, (void *) lines, add_line);
 
 	/* Sort on linenumber */
 	target_hash = HASH_OF(lines);
+#if PHP_VERSION_ID >= 70000
+	zend_hash_sort(target_hash, xdebug_lineno_cmp, 0 TSRMLS_CC);
+#else
 	zend_hash_sort(target_hash, zend_qsort, xdebug_lineno_cmp, 0 TSRMLS_CC);
+#endif
 
 	/* Add the branch and path info */
 	if (file->has_branch_info) {
-		MAKE_STD_ZVAL(file_info);
+		XDEBUG_MAKE_STD_ZVAL(file_info);
 		array_init(file_info);
 
-		MAKE_STD_ZVAL(functions);
+		XDEBUG_MAKE_STD_ZVAL(functions);
 		array_init(functions);
 
 		xdebug_hash_apply(file->functions, (void *) functions, add_cc_function);
 
-		add_assoc_zval_ex(file_info, "lines", 6, lines);
-		add_assoc_zval_ex(file_info, "functions", 10, functions);
+		add_assoc_zval_ex(file_info, "lines", HASH_KEY_SIZEOF("lines"), lines);
+		add_assoc_zval_ex(file_info, "functions", HASH_KEY_SIZEOF("functions"), functions);
 
-		add_assoc_zval_ex(retval, file->name, strlen(file->name) + 1, file_info);
+		add_assoc_zval_ex(retval, file->name, HASH_KEY_STRLEN(file->name), file_info);
+#if PHP_VERSION_ID >= 70000
+		efree(functions);
+		efree(file_info);
+#endif
 	} else {
-		add_assoc_zval_ex(retval, file->name, strlen(file->name) + 1, lines);
+		add_assoc_zval_ex(retval, file->name, HASH_KEY_STRLEN(file->name), lines);
 	}
+
+#if PHP_VERSION_ID >= 70000
+	efree(lines);
+#endif
 }
 
 PHP_FUNCTION(xdebug_get_code_coverage)
