@@ -243,11 +243,6 @@ zval *xdebug_get_zval(zend_execute_data *zdata, int node_type, const znode_op *n
 
 inline static HashTable *fetch_ht_from_zval(zval *z TSRMLS_DC)
 {
-#if PHP_VERSION_ID >= 70000
-	if (Z_TYPE_P(z) == IS_REFERENCE) {
-		z = &z->value.ref->val;
-	}
-#endif
 	switch (Z_TYPE_P(z)) {
 		case IS_ARRAY:
 			return Z_ARRVAL_P(z);
@@ -264,6 +259,13 @@ inline static char *fetch_classname_from_zval(zval *z, int *length, zend_class_e
 {
 	zend_string *class_name;
 
+	if (Z_TYPE_P(z) == IS_INDIRECT) {
+		z = z->value.zv;
+	}
+	if (Z_TYPE_P(z) == IS_REFERENCE) {
+		z = &z->value.ref->val;
+	}
+
 	if (Z_TYPE_P(z) != IS_OBJECT) {
 		return NULL;
 	}
@@ -405,6 +407,15 @@ static zval* fetch_zval_from_symbol_table(zval *parent, char* name, unsigned int
 	zend_property_info *zpp;
 
 	if (parent) {
+#if PHP_VERSION_ID >= 70000
+		if (Z_TYPE_P(parent) == IS_INDIRECT) {
+			parent = parent->value.zv;
+		}
+		if (Z_TYPE_P(parent) == IS_REFERENCE) {
+			parent = &parent->value.ref->val;
+		}
+#endif
+
 		ht = fetch_ht_from_zval(parent TSRMLS_CC);
 	}
 
@@ -590,7 +601,7 @@ static zval* fetch_zval_from_symbol_table(zval *parent, char* name, unsigned int
 				goto cleanup;
 			}
 			element_length = name_length;
-					
+
 			/* All right, time for a mega hack. It's SplObjectStorage access time! */
 			if (strncmp(ccn, "SplObjectStorage", ccnl) == 0 && strncmp(name, "storage", name_length) == 0) {
 				element = NULL;
@@ -648,6 +659,7 @@ cleanup:
 	if (element) {
 		free(element);
 	}
+
 	return retval_p;
 }
 
@@ -931,11 +943,11 @@ static int xdebug_array_element_export(zval **zv TSRMLS_DC, int num_args, va_lis
 		options->runtime[level].current_element_nr < options->runtime[level].end_element_nr)
 	{
 		if (HASH_KEY_IS_NUMERIC(hash_key)) { /* numeric key */
-			xdebug_str_add(str, xdebug_sprintf("%ld => ", HASH_APPLY_NUMERIC(hash_key)), 1);
+			xdebug_str_add(str, xdebug_sprintf(XDEBUG_INT_FMT " => ", HASH_APPLY_NUMERIC(hash_key)), 1);
 		} else { /* string key */
 			SIZETorINT newlen = 0;
 			char *tmp, *tmp2;
-			
+
 			tmp = xdebug_str_to_str((char*) HASH_APPLY_KEY_VAL(hash_key), HASH_APPLY_KEY_LEN(hash_key), "'", 1, "\\'", 2, &newlen);
 			tmp2 = xdebug_str_to_str(tmp, newlen - 1, "\0", 1, "\\0", 2, &newlen);
 			if (tmp) {
@@ -1354,11 +1366,11 @@ static int xdebug_array_element_export_text_ansi(zval **zv TSRMLS_DC, int num_ar
 		xdebug_str_add(str, xdebug_sprintf("%*s", (level * 2), ""), 1);
 
 		if (HASH_KEY_IS_NUMERIC(hash_key)) { /* numeric key */
-			xdebug_str_add(str, xdebug_sprintf("[%ld] %s=>%s\n", HASH_APPLY_NUMERIC(hash_key), ANSI_COLOR_POINTER, ANSI_COLOR_RESET), 1);
+			xdebug_str_add(str, xdebug_sprintf("[" XDEBUG_INT_FMT "] %s=>%s\n", HASH_APPLY_NUMERIC(hash_key), ANSI_COLOR_POINTER, ANSI_COLOR_RESET), 1);
 		} else { /* string key */
 			SIZETorINT newlen = 0;
 			char *tmp, *tmp2;
-			
+
 			tmp = xdebug_str_to_str((char*) HASH_APPLY_KEY_VAL(hash_key), HASH_APPLY_KEY_LEN(hash_key), "'", 1, "\\'", 2, &newlen);
 			tmp2 = xdebug_str_to_str(tmp, newlen - 1, "\0", 1, "\\0", 2, &newlen);
 			if (tmp) {
@@ -1404,14 +1416,14 @@ static int xdebug_object_element_export_text_ansi(zval **zv TSRMLS_DC, int num_a
 
 			modifier = xdebug_get_property_info((char*) HASH_APPLY_KEY_VAL(hash_key), HASH_APPLY_KEY_LEN(hash_key), &prop_name, &class_name);
 			xdebug_str_add(str, xdebug_sprintf("%s%s%s%s%s $%s %s=>%s\n",
-			               ANSI_COLOR_MODIFIER, ANSI_COLOR_BOLD, modifier, ANSI_COLOR_BOLD_OFF, ANSI_COLOR_RESET, 
+			               ANSI_COLOR_MODIFIER, ANSI_COLOR_BOLD, modifier, ANSI_COLOR_BOLD_OFF, ANSI_COLOR_RESET,
 			               prop_name, ANSI_COLOR_POINTER, ANSI_COLOR_RESET), 1);
 
 			xdfree(prop_name);
 			xdfree(class_name);
 		} else {
 			xdebug_str_add(str, xdebug_sprintf("%s%spublic%s%s ${%d} %s=>%s\n",
-			               ANSI_COLOR_MODIFIER, ANSI_COLOR_BOLD, ANSI_COLOR_BOLD_OFF, ANSI_COLOR_RESET, 
+			               ANSI_COLOR_MODIFIER, ANSI_COLOR_BOLD, ANSI_COLOR_BOLD_OFF, ANSI_COLOR_RESET,
 			               HASH_APPLY_NUMERIC(hash_key), ANSI_COLOR_POINTER, ANSI_COLOR_RESET), 1);
 		}
 		xdebug_var_export_text_ansi(zv, str, mode, level + 1, debug_zval, options TSRMLS_CC);
@@ -1439,7 +1451,7 @@ void xdebug_var_export_text_ansi(zval **struc, xdebug_str *str, int mode, int le
 	if (!struc || !(*struc)) {
 		return;
 	}
-	
+
 	xdebug_str_add(str, xdebug_sprintf("%*s", (level * 2) - 2, ""), 1);
 
 #if PHP_VERSION_ID >= 70000
@@ -1789,7 +1801,7 @@ static int object_item_add_to_merged_hash(zval **zv TSRMLS_DC, int num_args, va_
 		item->name_len = HASH_APPLY_KEY_LEN(hash_key) - 1;
 		item->index = hash_key->h;
 	} else {
-		item->name = xdebug_sprintf("%ld", index);
+		item->name = xdebug_sprintf(XDEBUG_INT_FMT, index);
 		item->name_len = strlen(item->name);
 	}
 #else
@@ -1881,7 +1893,7 @@ static int xdebug_array_element_export_xml_node(zval **zv TSRMLS_DC, int num_arg
 		options->runtime[level].current_element_nr < options->runtime[level].end_element_nr)
 	{
 		node = xdebug_xml_node_init("property");
-	
+
 		if (!HASH_KEY_IS_NUMERIC(hash_key)) { /* string key */
 			name = xdstrndup(HASH_APPLY_KEY_VAL(hash_key), HASH_APPLY_KEY_LEN(hash_key));
 			name_len = HASH_APPLY_KEY_LEN(hash_key) - 1;
@@ -1892,7 +1904,7 @@ static int xdebug_array_element_export_xml_node(zval **zv TSRMLS_DC, int num_arg
 				xdebug_str_addl(&full_name, "']", 2, 0);
 			}
 		} else {
-			name = xdebug_sprintf("%ld", HASH_APPLY_NUMERIC(hash_key));
+			name = xdebug_sprintf(XDEBUG_INT_FMT, HASH_APPLY_NUMERIC(hash_key));
 			name_len = strlen(name);
 			if (parent_name) {
 				xdebug_str_add(&full_name, xdebug_sprintf("%s[%s]", parent_name, name), 1);
@@ -1963,10 +1975,10 @@ static int xdebug_object_element_export_xml_node(xdebug_object_item **item TSRML
 		} else { /* Numerical property name */
 			modifier = "public";
 
-			xdebug_xml_add_attribute_ex(node, "name", xdebug_sprintf("%ld", (*item)->index), 0, 1);
+			xdebug_xml_add_attribute_ex(node, "name", xdebug_sprintf(XDEBUG_INT_FMT, (*item)->index), 0, 1);
 
 			if (parent_name) {
-				full_name = xdebug_sprintf("%s%s%ld", parent_name, (*item)->type == XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY ? "::" : "->", (*item)->index);
+				full_name = xdebug_sprintf("%s%s" XDEBUG_INT_FMT, parent_name, (*item)->type == XDEBUG_OBJECT_ITEM_TYPE_STATIC_PROPERTY ? "::" : "->", (*item)->index);
 				xdebug_xml_add_attribute_ex(node, "fullname", full_name, 0, 1);
 			}
 		}
@@ -2358,7 +2370,7 @@ static int xdebug_array_element_export_fancy(zval **zv TSRMLS_DC, int num_args,
 		xdebug_str_add(str, xdebug_sprintf("%*s", (level * 4) - 2, ""), 1);
 
 		if (HASH_KEY_IS_NUMERIC(hash_key)) { /* numeric key */
-			xdebug_str_add(str, xdebug_sprintf("%ld <font color='%s'>=&gt;</font> ", HASH_APPLY_NUMERIC(hash_key), COLOR_POINTER), 1);
+			xdebug_str_add(str, xdebug_sprintf(XDEBUG_INT_FMT " <font color='%s'>=&gt;</font> ", HASH_APPLY_NUMERIC(hash_key), COLOR_POINTER), 1);
 		} else { /* string key */
 			xdebug_str_addl(str, "'", 1, 0);
 			tmp_str = xdebug_xmlize((char*) HASH_APPLY_KEY_VAL(hash_key), HASH_APPLY_KEY_LEN(hash_key) - 1, &newlen);
@@ -2879,7 +2891,7 @@ char* xdebug_show_fname(xdebug_func f, int html, int flags TSRMLS_DC)
 				);
 			}
 			break;
-		}	
+		}
 
 		case XFUNC_EVAL:
 			return xdstrdup("eval");
@@ -2901,8 +2913,11 @@ char* xdebug_show_fname(xdebug_func f, int html, int flags TSRMLS_DC)
 			return xdstrdup("require_once");
 			break;
 
+		case XFUNC_ZEND_PASS:
+			return xdstrdup("{zend_pass}");
+			break;
+
 		default:
 			return xdstrdup("{unknown}");
 	}
 }
-
